==> decoupled_toolbox-1.0.1-rc1.tar.gz: decoupled_toolbox/src/Plugin/Field/FieldFormatter/EntityReferenceDecoupledFormatterBase.php <==
<?php

namespace Drupal\decoupled_toolbox\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\decoupled_toolbox\Exception\UnexpectedFormatterException;
use Drupal\decoupled_toolbox\FieldValueAndOptions;

/**
 * Base plugin implementation of the decoupled entity formatters.
 */
abstract class EntityReferenceDecoupledFormatterBase extends DecoupledFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElementsForDecoupled(FieldItemListInterface $items, $langCode): FieldValueAndOptions {
    $this->validateSettingsOnRender();

    $decoupledFieldKey = $this->getDecoupledFieldKey();
    $decoupledFieldLocation = $this->getDecoupledFieldLocation();

    if ($this->isMultivaluedFieldItemList($items)) {
      $processedMultiValuedItems = $this->viewMultivaluedFieldItemList($items);

      if (empty($processedMultiValuedItems)) {
        // No value set.
        $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
        $this->setProcessedFieldValue($fieldValue);
        return $fieldValue;
      }

      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, $processedMultiValuedItems, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    try {
      /* @var \Drupal\Core\Field\FieldItemInterface $item */
      $item = $items->first();
    }
    catch (MissingDataException $exception) {
      $this->getLogger('decoupled_toolbox')->error($exception->getMessage());
      // No value set.
      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    if (empty($item)) {
      // No value set.
      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    $fieldValue = new FieldValueAndOptions($decoupledFieldKey, $this->viewFieldItem($item), $decoupledFieldLocation);
    $this->setProcessedFieldValue($fieldValue);
    return $fieldValue;
  }

  /**
   * {@inheritdoc}
   */
  protected function viewMultivaluedFieldItemList(FieldItemListInterface $items): array {
    if (!$items instanceof EntityReferenceFieldItemListInterface) {
      throw new UnexpectedFormatterException('Tried to render an entity field list, but the field does not implement EntityReferenceFieldItemListInterface.');
    }

    $data = [];

    foreach ($items as $delta => $item) {
      $data[$delta] = $this->viewFieldItem($item);
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public static function getOutputDefinitions() {
    return [
    ];
  }

}

==> decoupled_toolbox-1.5.1-rc10.tar.gz: decoupled_toolbox/src/Plugin/Field/FieldFormatter/EntityReferenceDecoupledFormatterBase.php <==
<?php

namespace Drupal\decoupled_toolbox\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\decoupled_toolbox\Exception\InvalidContentException;
use Drupal\decoupled_toolbox\Exception\UnexpectedFormatterException;
use Drupal\decoupled_toolbox\FieldValueAndOptions;

/**
 * Base plugin implementation of the decoupled entity formatters.
 */
abstract class EntityReferenceDecoupledFormatterBase extends DecoupledFormatterBase {

  /**
   * Checks if missing entity reference should be ignored.
   *
   * @return bool
   *   TRUE if missing entity reference should be ignored, FALSE otherwise.
   */
  protected function ignoreMissingEntityReference(): bool {
    return $this->getDecoupledToolboxConfig('ignore_missing_entity_reference', FALSE);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElementsForDecoupled(FieldItemListInterface $items, $langCode): FieldValueAndOptions {
    $this->validateSettingsOnRender();

    $decoupledFieldKey = $this->getDecoupledFieldKey();
    $decoupledFieldLocation = $this->getDecoupledFieldLocation();

    if ($this->getDecoupledForcedMultipleOutput() || $this->isMultivaluedFieldItemList($items)) {
      $processedMultiValuedItems = $this->viewMultivaluedFieldItemList($items);

      if (empty($processedMultiValuedItems)) {
        // No value set.
        $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
        $this->setProcessedFieldValue($fieldValue);
        return $fieldValue;
      }

      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, $processedMultiValuedItems, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    try {
      /** @var \Drupal\Core\Field\FieldItemInterface $item */
      $item = $items->first();
    }
    catch (MissingDataException $exception) {
      $this->getLogger('decoupled_toolbox')->error($exception->getMessage());
      // No value set.
      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    if (empty($item)) {
      // No value set.
      $fieldValue = new FieldValueAndOptions($decoupledFieldKey, NULL, $decoupledFieldLocation);
      $this->setProcessedFieldValue($fieldValue);
      return $fieldValue;
    }

    $fieldValue = new FieldValueAndOptions($decoupledFieldKey, $this->viewFieldItem($item), $decoupledFieldLocation);
    $this->setProcessedFieldValue($fieldValue);
    return $fieldValue;
  }

  /**
   * {@inheritdoc}
   */
  protected function viewMultivaluedFieldItemList(FieldItemListInterface $items): array {
    if (!$items instanceof EntityReferenceFieldItemListInterface) {
      throw new UnexpectedFormatterException('Tried to render an entity field list, but the field does not implement EntityReferenceFieldItemListInterface.');
    }

    $data = [];

    foreach ($items as $item) {
      try {
        $data[] = $this->viewFieldItem($item);
      }
      catch (InvalidContentException $exception) {
        // This happens when the reference was deleted.
        if ($this->ignoreMissingEntityReference()) {
          continue;
        }

        // Rethrow the exception.
        throw $exception;
      }
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public static function getOutputDefinitions() {
    return [];
  }

}
